#ifndef CONNECTION_HPP
#define CONNECTION_HPP

//****************************************************************************************************
//*                                              Includes                                            *
//****************************************************************************************************

#include "Other.hpp"

using namespace System;
using namespace System::Net;
using namespace System::Net::Sockets;

//****************************************************************************************************
//*                                       Declaration of the Class                                   *
//****************************************************************************************************
//name: Connection
//description: The class responsibles for the undisturbed function of the network contacts.
//----------------------------------------------------------------------------------------------------

ref class Connection{
	private:
		TcpClient^ client;
	
	public:
		Connection() {}
		bool Connect(String^ address);
		void Disconnect();
		bool sendMSG(String^ str);
		String^ getMSG();
		String^ waitMSG();
};

//****************************************************************************************************
//*                                      Definition of the Functions                                 *
//****************************************************************************************************

//----------------------------------------------------------------------------------------------------
//name: Connect()
//return: bool
//parameters: System::String^ (address)
//description: The function tries to build up the connection with the server at the given ip address. If 
// it success, return true else return false.
//----------------------------------------------------------------------------------------------------
bool Connection::Connect(String^ address){
	try{
	client = gcnew TcpClient(address,11111); 
	sendMSG("HE$");
	return true;
	}catch (Exception^ ex){
		ex; 
		if (client != nullptr) client->Close();
        client=nullptr;
		return false;
	}
}

//----------------------------------------------------------------------------------------------------
//name: Disconnect()
//return: -
//parameters: -
//description: The function breaks the connection with the server.
//----------------------------------------------------------------------------------------------------
void Connection::Disconnect(){
	try{
	if (client==nullptr) throw gcnew IO::IOException();
	sendMSG("BY$"); 
	Other::wait(0.1);
	client->Close();
	client=nullptr;
	}catch (Exception^ ex) {ex;}
}

//----------------------------------------------------------------------------------------------------
//name: sendMSG()
//return: bool
//parameters: System::String^(str)
//description: The function sends the given String to the server. If an error occurs, return false 
// else true.
//----------------------------------------------------------------------------------------------------
bool Connection::sendMSG(String^ str){
	try{
	if (client==nullptr) throw gcnew IO::IOException();
	array<Byte>^ data = System::Text::Encoding::ASCII->GetBytes(str);
	data->Clear(data,data->Length-1,1);
	client->GetStream()->Write(data,0,data->Length);
	client->GetStream()->Flush();
	return true;
	}catch (IO::IOException^ ex){
		ex;
		client->Close();
		client=nullptr;
		return false;
	}
}

//----------------------------------------------------------------------------------------------------
//name: getMSG()
//return: System::String^
//parameters: -
//description: The function receives message from the server. If no message arrived, return null.
//----------------------------------------------------------------------------------------------------
String^ Connection::getMSG(){
	if (client==nullptr) throw gcnew IO::IOException();
	array<Byte>^ bytes=gcnew array<Byte>(256);
	Int32 i=0;
	i=client->GetStream()->Read(bytes,0,bytes->Length);
	return System::Text::Encoding::ASCII->GetString(bytes,0,i);
}

//----------------------------------------------------------------------------------------------------
//name: waitMSG()
//return: System::String^
//parameters: -
//description: The function receives message from the server. If no message arrived, wait for it one 
// second. If no message arrived, return a specific String TIMEOUT.
//----------------------------------------------------------------------------------------------------
String^ Connection::waitMSG(){
	if (client==nullptr) throw gcnew IO::IOException();
	array<Byte>^ bytes=gcnew array<Byte>(256);
	Int32 i=0; Int32 counter=0;
	while (client->Available == 0){
		++counter;
		Other::wait(0.01);
		if (counter>100) return gcnew String("TIMEOUT");
	}
	i=client->GetStream()->Read(bytes,0,bytes->Length);
	return System::Text::Encoding::ASCII->GetString(bytes,0,i);
}



#endif