#ifndef OBJECTSEARCHER_HPP
#define OBJECTSEARCHER_HPP

#include "PlotterObject.hpp"
#include <vector>
#include <fstream>
#include <string>
#include <cmath>

#define default_WebCamH 330;

#define mm_pixel 0.54 //0.556 - %

const int onlyX[5]={319,299,319,339,319};
const int onlyY[5]={239,239,219,239,259};
const int demoboundstart[4]={319,219,319,219};
const float degree=3.141593f/180.0f;

using namespace System::Drawing;
using namespace System;

ref class ObjectSearcher{
	private: 
		static const char* scan_filename="scan.txt";
		static const char* detect_filename="detect.txt";
		static const char* file_config="Config.txt";
		std::vector<PlotterObject*>* foundedObject;
		std::vector<PlotterObject*>* candidates;
		int* demobound;

	public:
		int offset_WebCamX;
		int offset_WebCamY;
		int offset_WebCamH;
		int WsCordX;
		int WsCordY;
		int WsSizeX;
		int WsSizeY;

	private:
		bool _isIn(int x,int y); 
		void _remove(Bitmap^ img,int x,int y,int xi,int yi);
		bool _isWorkSpace(Color color);
		Color _ToBin(Color color);
		int _black(Bitmap^ img,int x, int y);
		void Remove(Bitmap^ img);
		void Brightness(Bitmap^ img,int i);
		void ToColorless(Bitmap^ img);
		void DetectObjects(Bitmap^ img,Coord coord);
		void OnlyOne(Bitmap^ img,int* bound,int* T);
		void CheckCandidates();
		Color IdentColor(Bitmap^ img);
		PlotterObject* Identify(int* bound, int A,int B,int T,int R,Color color, Coord coord,bool* ok);
		int Get(Bitmap^ img,int* bound,int *A,int* B);
		int distance(int x1,int y1, int x2, int y2);

	public:
		ObjectSearcher();
		void Clear();
		bool isValid();
		Bitmap^ Scan(Bitmap^ img,Coord coord);
		Bitmap^ Detect(Bitmap^ img,Coord coord);
		PlotterObject getCandidate(int i);
		int canNumber() {return candidates->size();}
		String^ getObject(int i);
		int number() {return foundedObject->size();}
		Bitmap^ DemoDetect(Bitmap^ img,int i,String ^step);
		Bitmap^ DemoScan(Bitmap^ img,int i,String^ step);
};

ObjectSearcher::ObjectSearcher(){
	demobound=new int();
	candidates= new std::vector<PlotterObject*>();
	foundedObject= new std::vector<PlotterObject*>();
	//Default offset values
	offset_WebCamY=5;
	offset_WebCamX=-65;
	offset_WebCamH=default_WebCamH;
	WsCordX=0;
	WsCordY=0;
	WsSizeX=300;
	WsSizeY=400;
	//Config file read
	std::ifstream file(file_config);
	std::string str;
	while (getline(file,str)){
		if (str.c_str()[0]=='%') continue;
		if (str.substr(0,7)=="WebCamX") offset_WebCamX=atoi(str.substr(7).c_str());
		if (str.substr(0,7)=="WebCamY") offset_WebCamY=atoi(str.substr(7).c_str());
		if (str.substr(0,7)=="WebCamH") offset_WebCamH=atoi(str.substr(7).c_str());
		if (str.substr(0,7)=="WsCordX") WsCordX=atoi(str.substr(7).c_str());
		if (str.substr(0,7)=="WsCordY") WsCordY=atoi(str.substr(7).c_str());
		if (str.substr(0,7)=="WsSizeX") WsSizeX=atoi(str.substr(7).c_str());
		if (str.substr(0,7)=="WsSizeY") WsSizeY=atoi(str.substr(7).c_str());
	}
	file.close();

}

void ObjectSearcher::Clear(){
	candidates->clear();
	foundedObject->clear();
}

bool ObjectSearcher::isValid(){
	return (foundedObject->size() > 0);
}

String^ ObjectSearcher::getObject(int i){
	if ((unsigned int)i > foundedObject->size()-1) return L"ERROR";
	else return (i.ToString()+" "+foundedObject->at(i)->toSend());
}

bool ObjectSearcher::_isWorkSpace(Color color){
	return ((color.R+color.G+color.B)/3>128)&&(color.B>160);

}

bool ObjectSearcher::_isIn(int x,int y){ 
	return (x>=0 && x<640 && y>=0 && y<480);
}

int ObjectSearcher::_black(Bitmap^ img,int x, int y){
	int counter=0;
	for (int i=-1;i<2;++i)
		for (int j=-1;j<2;++j){
			if (_isIn(x+i,y+i) && img->GetPixel(x+i,y+j).ToArgb()==Color::Black.ToArgb()) ++counter;
			if (_isIn(x+i,y+i) && img->GetPixel(x+i,y+j).ToArgb()==Color::LightGray.ToArgb()) ++counter;
		}
	return counter;
}

void ObjectSearcher::_remove(Bitmap^ img,int x,int y,int xi,int yi)
{
 if (!_isIn(x,y)) return;
 if (!_isWorkSpace(img->GetPixel(x,y)))
  {
   img->SetPixel(x,y,Color::White);
   _remove(img,x+xi,y,xi,yi);
   _remove(img,x,y+yi,xi,yi);
  }
}

void ObjectSearcher::Remove(System::Drawing::Bitmap^ img)
{
 int threshold=25;
 if (!_isWorkSpace(img->GetPixel(  0,  0))) _remove(img,0,0,1,1);
 if (!_isWorkSpace(img->GetPixel(639,479))) _remove(img,639,479,-1,-1);
 if (!_isWorkSpace(img->GetPixel(639,  0))) _remove(img,639,0,-1,1);
 if (!_isWorkSpace(img->GetPixel(  0,479))) _remove(img,0,479,1,-1);
}

void ObjectSearcher::Brightness(Bitmap^ img,int i){
	for (int y=0;y<480;++y){
		for (int x=0;x<640;++x){
			Color c=img->GetPixel(x,y);
			int r =(int)(c.R*(1.0+(double)i/100.0));
			int g =(int)(c.G*(1.0+(double)i/100.0));
			int b =(int)(c.B*(1.0+(double)i/100.0));
			if (r>255) r=255;
			if (g>255) g=255;
			if (b>255) b=255;
			img->SetPixel(x,y,Color::FromArgb(r,g,b));
		}
	}
}

void ObjectSearcher::ToColorless(Bitmap^ img){
	for (int y=0;y<480;++y){
		for (int x=0;x<640;++x){
			img->SetPixel(x,y,_ToBin(img->GetPixel(x,y)));
		}
	}
}
void ObjectSearcher::DetectObjects(Bitmap^ img,Coord coord){
	std::vector<Coord*>* tmp=new std::vector<Coord*>();
	std::vector<Coord*>* points=new std::vector<Coord*>();
	for (int y=0;y<480;++y){
		for (int x=0;x<640;++x){
			
			if (img->GetPixel(x,y).ToArgb()==Color::Black.ToArgb()) {
				tmp->push_back(new Coord(x,y));
			}		
			else continue;
			while (!tmp->empty()){
				Coord* c=tmp->front(); 
				if (img->GetPixel(c->getX(),c->getY()).ToArgb()!=Color::Black.ToArgb()) {tmp->erase(tmp->begin());continue;}			
				img->SetPixel(c->getX(),c->getY(),Color::Red);
				points->push_back(c);
				tmp->erase(tmp->begin());
				if (c->getX()>0 && img->GetPixel(c->getX()-1,c->getY()).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX()-1,c->getY()));
				if (c->getX()<639 &&img->GetPixel(c->getX()+1,c->getY()).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX()+1,c->getY()));
				if (c->getY()>0 &&img->GetPixel(c->getX(),c->getY()-1).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX(),c->getY()-1));
				if (c->getY()<479 &&img->GetPixel(c->getX(),c->getY()+1).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX(),c->getY()+1));
			}
			int tmpx=0; 
			int tmpy=0;
			for (unsigned int i=0;i<points->size();++i){			
				if (points->size()<400){
					img->SetPixel(points->at(i)->getX(),points->at(i)->getY(),Color::LightGray);
				}
				else{
					img->SetPixel(points->at(i)->getX(),points->at(i)->getY(),Color::LightGray);
					tmpx+=points->at(i)->getX();
					tmpy+=points->at(i)->getY();
				}
			}
			int tx=tmpx/points->size();
			int ty=tmpy/points->size();
			img->SetPixel(tx,ty,Color::Blue);	
			if (tx>20 && tx<620 && ty>20 && ty<450){
				img->SetPixel(tx,ty,Color::Red);		
				int X=(int)((ty-240)*mm_pixel+offset_WebCamX)+coord.getX();
				int Y=(int)((320-tx)*mm_pixel+offset_WebCamY)+coord.getY();

				if (X < WsCordX || X > WsCordX+WsSizeX || Y < WsCordY || Y > WsCordY+WsSizeY) {img->SetPixel(tx,ty,Color::Yellow);} 
				else if (coord.getX()!=0 && coord.getY()!=0)
					candidates->push_back(new PlotterObject(Coord(X,Y),-1,-1));
			}
			points->clear();
	
	
		}
	}
}

Color ObjectSearcher::_ToBin(Color color){
	if (color.R>128 && color.G>128 && color.B>128) return Color::White; else return Color::Black;
}

void ObjectSearcher::CheckCandidates(){
	if (candidates->size()<2) return;
	for (unsigned int c1=0;c1<candidates->size();++c1)
	{
		for (unsigned int c2=c1+1;c2<candidates->size();++c2){
			if (Coord::distance(candidates->at(c1)->getCoord(),candidates->at(c2)->getCoord()) < 100 ){
				candidates->erase(candidates->begin()+c2);
			}
		}
	}
}

void ObjectSearcher::OnlyOne(Bitmap^ img,int* bound,int* T){
	std::vector<Coord*>* tmp=new std::vector<Coord*>();
	int counter = 0;

	for (int i=0;i<5;++i){
		if (img->GetPixel(onlyX[i],onlyY[i]).ToArgb()==Color::Black.ToArgb()) {tmp->push_back(new Coord(onlyX[i],onlyY[i])); break;}
	}

	while (!tmp->empty()){
		Coord* c=tmp->front(); 
		if (c->getX() < bound[0]) bound[0]=c->getX();
		if (c->getX() > bound[2]) bound[2]=c->getX();
		if (c->getY() < bound[1]) bound[1]=c->getY();
		if (c->getY() > bound[3]) bound[3]=c->getY();
		if (img->GetPixel(c->getX(),c->getY()).ToArgb()!=Color::Black.ToArgb()) {tmp->erase(tmp->begin());continue;}			
		img->SetPixel(c->getX(),c->getY(),Color::Red);
		++counter;
		tmp->erase(tmp->begin());
		if (c->getX()>0 && img->GetPixel(c->getX()-1,c->getY()).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX()-1,c->getY()));
		if (c->getX()<639 &&img->GetPixel(c->getX()+1,c->getY()).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX()+1,c->getY()));
		if (c->getY()>0 &&img->GetPixel(c->getX(),c->getY()-1).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX(),c->getY()-1));
		if (c->getY()<479 &&img->GetPixel(c->getX(),c->getY()+1).ToArgb()==Color::Black.ToArgb()) tmp->push_back(new Coord(c->getX(),c->getY()+1));
	}

	*T=counter;
	for (int y=0;y<480;++y)
		for (int x=0;x<640;++x){
			if (img->GetPixel(x,y).ToArgb()==Color::Red.ToArgb()) img->SetPixel(x,y,Color::LightGray); else img->SetPixel(x,y,Color::White);
		}
}

int ObjectSearcher::Get(Bitmap^ img,int* bound,int* A,int *B){
	
	int cornerX[]={0,0,0,0};
	int cornerY[]={0,0,0,0};
	int counter=0;
	double ret;

	for (int x=bound[0];x<bound[2];++x)
		if (img->GetPixel(x,bound[1]).ToArgb()==Color::LightGray.ToArgb()){
			cornerX[0]+=x;
			counter++;
		}
	if (counter ==0 ) return -999;
	cornerX[0]=cornerX[0]/counter;
	cornerY[0]=bound[1];

	counter=0;
	for (int x=bound[0];x<bound[2];++x)
		if (img->GetPixel(x,bound[3]).ToArgb()==Color::LightGray.ToArgb()){
			cornerX[2]+=x;
			counter++;
		}
	if (counter ==0 ) return -999;
	cornerX[2]=cornerX[2]/counter;
	cornerY[2]=bound[3];

	counter=0;
	for (int y=bound[1];y<bound[3];++y)
		if (img->GetPixel(bound[0],y).ToArgb()==Color::LightGray.ToArgb()){
			cornerY[1]+=y;
			counter++;
		}
	if (counter ==0 ) return -999;
	cornerY[1]=cornerY[1]/counter;
	cornerX[1]=bound[0];
	
	counter=0;
	for (int y=bound[1];y<bound[3];++y)
		if (img->GetPixel(bound[2],y).ToArgb()==Color::LightGray.ToArgb()){
			cornerY[3]+=y;
			counter++;
		}
	if (counter ==0 ) return -999;
	cornerY[3]=cornerY[3]/counter;
	cornerX[3]=bound[2];

	*A=(distance(cornerX[0],cornerY[0],cornerX[1],cornerY[1])+distance(cornerX[2],cornerY[2],cornerX[3],cornerY[3]))/2;
	*B=(distance(cornerX[1],cornerY[1],cornerX[2],cornerY[2])+distance(cornerX[3],cornerY[3],cornerX[0],cornerY[0]))/2;
	
	double r1,r2;

	if (*A>*B){
		r1=atan2((double)(cornerY[1]-cornerY[0]),(double)(cornerX[1]-cornerX[0]));
		r2=atan2((double)(cornerY[2]-cornerY[3]),(double)(cornerX[2]-cornerX[3]));
		ret=(r1+r2)/2;
	}
	else{
		r1=atan2((double)(cornerY[2]-cornerY[1]),(double)(cornerX[2]-cornerX[1]));
		r2=atan2((double)(cornerY[3]-cornerY[0]),(double)(cornerX[3]-cornerX[0]));
		ret=(r1+r2)/2;
	}
	for (unsigned int i=0; i<4;++i) img->SetPixel(cornerX[i],cornerY[i],Color::Red);

	return (int)((ret)/degree);

}

Color ObjectSearcher::IdentColor(Bitmap^ img){
	int red=0; int green=0; int blue=0; int counter=1;
	for (int y=230;y<251;++y)
		for (int x=310;x<331;++x)
			if ( _ToBin(img->GetPixel(x,y)).ToArgb()==Color::Black.ToArgb()){
				red+=img->GetPixel(x,y).R;
				green+=img->GetPixel(x,y).G;
				blue+=img->GetPixel(x,y).B;
				++counter;
			}
	return Color::FromArgb(red/counter,green/counter,blue/counter);
}

PlotterObject* ObjectSearcher::Identify(int* bound,int A,int B,int T, int R,Color color,Coord coord,bool* ok){
	PlotterObject* ret=new PlotterObject();
	int X=(int)(((bound[1]+bound[3])/2-240)*mm_pixel+offset_WebCamX)+coord.getX();
	int Y=(int)((320-(bound[0]+bound[2])/2)*mm_pixel+offset_WebCamY)+coord.getY();
	ret->setCoord(*new Coord(X,Y));
	if (color.R>color.G+color.B && color.G<100 && color.B<100) ret->setColor(1); //RED
	else if (color.G>color.R+color.B && color.R<100 && color.B<100) ret->setColor(2); //GREEN
	else if (color.B>color.R && color.B>color.G) ret->setColor(3); //BLUE
	else if (color.R>100 && color.G>100 && color.B < 100) ret->setColor(4); //Yellow
	int a=0; int b=0;
	if (A>B) {b=A; a=B;} else {a=A; b=B;}

	if (abs(a-16.7)<10 && abs(b-33.3)<10) {ret->setBaseType( 1); ret->setType(-2); ret->setOrientation( 1); *ok=false;}
	if (abs(a-33.3)<10 && abs(b-33.3)<10) {ret->setBaseType(-2); ret->setType(-2); ret->setOrientation(-2); *ok=false;}
	if (abs(a-16.7)<10 && abs(b-66.7)<10) {ret->setBaseType( 1); ret->setType( 2); ret->setOrientation( 2); *ok=true;}
	if (abs(a-33.3)<10 && abs(b-66.7)<10) {ret->setBaseType( 1); ret->setType( 2); ret->setOrientation( 3); *ok=true;}
	if (abs(a-16.6)<10 && abs(b- 100)<10) {ret->setBaseType( 1); ret->setType( 3); ret->setOrientation( 2); *ok=true;}
	if (abs(a-33.3)<10 && abs(b- 100)<10) {ret->setBaseType( 1); ret->setType( 3); ret->setOrientation( 3); *ok=true;}

	ret->setRotation(R);
	return ret;
}

Bitmap^ ObjectSearcher::Scan(Bitmap^ img,Coord coord){
	
	Brightness(img,40);
	Remove(img);
	ToColorless(img);
	DetectObjects(img,coord);
	CheckCandidates();

	std::ofstream file;
	file.open(scan_filename);
	for (unsigned int i=0;i<candidates->size();++i){
		file << (const char*)(System::Runtime::InteropServices::Marshal::StringToHGlobalAnsi(candidates->at(i)->toStr())).ToPointer()<<std::endl;
	}
	file.close();

	return img;
}

Bitmap^ ObjectSearcher::DemoScan(Bitmap^ img,int i,String^ step){
	switch(i){
		case 1 : Brightness(img,40); step=gcnew String("Modify Brightness"); break;
		case 2 : Remove(img); step=gcnew String("Remove OutArea");break;
		case 3 : ToColorless(img); step=gcnew String("Remove Colors"); break;
		case 4 : DetectObjects(img,Coord(150,200)); step=gcnew String("Detet Objects"); break;
	}
	return img;
}

int ObjectSearcher::distance(int x1,int y1, int x2, int y2){
	return (int)sqrt((double)((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)));
}
PlotterObject ObjectSearcher::getCandidate(int i){
	return *(candidates->at(i));
}

Bitmap^ ObjectSearcher::DemoDetect(Bitmap^ img,int i,String^ step){
	if (i==1) for (int t=0;t<4;++t) demobound[t]=demoboundstart[t];
	int *tmp=new int(0);
	switch (i){
		case 1 : Brightness(img,40); step=gcnew String("Modify Brightness"); break;
		case 2 : Remove(img); step=gcnew String("Remove OutArea"); break;
		case 3 : ToColorless(img); step=gcnew String("Remove Colors"); break;
		case 4 : OnlyOne(img,demobound,tmp); step=gcnew String("Search Middle Element"); break;
		case 5 : Get(img,demobound,new int(),new int()); step=gcnew String("Search Middle Element"); break;
	}
	std::ofstream f;
	f.open("ize.txt");
	f << demobound[0] << " "<< demobound[1] << " "<< demobound[2] << " "<< demobound[3] << std::endl;
	f.close();
	return img;
}

Bitmap^ ObjectSearcher::Detect(Bitmap^ img,Coord coord){
	int *T=new int(0);
	int *A=new int(0);
	int *B=new int(0);
	int R;
	bool* ok=new bool(true);
	int bound[4]={319,219,319,219};
	Brightness(img,40);
	Remove(img);
	Color color=IdentColor(img);
	ToColorless(img);
	OnlyOne(img,bound,T);
	R=Get(img,bound,A,B);

	PlotterObject* tmp=Identify(bound,(int)(*A*mm_pixel),(int)(*B*mm_pixel),(int)(*T*mm_pixel*mm_pixel),R,color,coord,ok);
	foundedObject->push_back(tmp);

	std::ofstream f;
	f.open(detect_filename);
	for (unsigned int i=0;i<foundedObject->size();++i){
		f << (const char*)(System::Runtime::InteropServices::Marshal::StringToHGlobalAnsi(foundedObject->at(i)->toSend())).ToPointer()<<std::endl;
	}
	f.close();
	return img;
}

#endif