#include "m5apiw32.h"
#include <time.h>
#include <vector>
#include <fstream>

int ret = 0; // ltalnos visszatrsi rtk. Nem hasznljuk semmire.
char pInitString[] = "ESD:0,250"; // Inicializl sztring a Stage indtshoz

const int ID[6]={0,1,2,3,4,23};

// PowerCube osztly
class PCube 
{
	int id;
	bool inverted;
	
public:
	// ID functions
	int getId();

	// Homing functions
	void setHomeVel(float);
	float getHomeVel();
	void goHome();
		
	// Halt function
	void halt();

	// Reset function
	void reset();
	
	// State functions
	unsigned long getModuleState();

	// Inversion functions
	void setInversion(bool);
	bool getInversion();

	//
	// Current functions
	//
	void setMaxCurr(float); // constraints
	float getMaxCurr();		
	
	float getCurrent();
	void moveCur(float);
	
	//
	// Position
	//
	void setMaxPos(float); // constraints
	float getMaxPos();
	void setMinPos(float);
	float getMinPos();

	float getActPos();
	void movePos(float); // centimterben megadott koordinta
	void movePosExtended(float, float, float);

	//
	// Velocity
	//
	float getMaxVel();
	void setMaxVel(float);

	float getVelocity();
	void moveVel(float);

	//
	// Acceleration
	//
	void setMaxAcc(float);
	float getMaxAcc();
	
	

	PCube(int id);  // konstruktor
	~PCube(); // destruktor
};

void PCube::moveVel(float value)
{
	ret = PCube_moveVel(0, this->id, value);
}

void PCube::movePosExtended(float pos, float vel, float acc)
{
	ret = PCube_moveRamp(0, this->id, pos, vel, acc);
}

void PCube::setMaxAcc(float value)
{
	ret = PCube_setMaxAcc(0, this->id, value);
}

float PCube::getMaxAcc()
{
	float temp;
	ret = PCube_getMaxAcc(0, this->id, &temp);
	return temp;
}

void PCube::setMaxVel(float value)
{
	ret = PCube_setMaxVel(0, this->id, value);
}

float PCube::getMaxVel()
{
	float temp;
	ret = PCube_getMaxVel(0, this->id, &temp);
	return temp;
}

float PCube::getMinPos()
{
	float temp;
	ret = PCube_getMinPos(0, this->id, &temp);
	return temp;
}

float PCube::getMaxPos()
{
	float temp;
	ret = PCube_getMaxPos(0, this->id, &temp);
	return temp;
}

void PCube::setMaxPos(float value)
{
	ret = PCube_setMaxPos(0, this->id, value);
}

void PCube::setMinPos(float value)
{
	ret = PCube_setMinPos(0, this->id, value);
}

void PCube::moveCur(float value)
{
	ret = PCube_moveCur(0, this->id, value);
}

bool PCube::getInversion()
{
	return this->inverted;
}

void PCube::setInversion(bool value)
{
	this->inverted = value;
}

void PCube::movePos(float pos)
{
	ret = PCube_movePos(0, this->id, (this->inverted) ? -1*pos : pos);
}

void PCube::reset()
{
	ret=PCube_resetModule(0,this->id);
}

void PCube::halt()
{
	ret=PCube_haltModule(0,this->id); 
}

unsigned long PCube::getModuleState()
{
	unsigned long temp;
	ret=PCube_getModuleState(0,this->id,&temp);
	return temp;
}

void PCube::goHome()
{
	ret=PCube_homeModule(0,this->id);
}

float PCube::getCurrent() 
{
	float temp;
	ret=PCube_getCur(0,this->id,&temp);
	return temp;
}

void PCube::setHomeVel(float homeVel)
{
	ret=PCube_setHomeVel(0,this->id,homeVel);
}

float PCube::getHomeVel()
{
	float temp;
	ret = PCube_getHomeVel(0, this->id, &temp);
	return temp;
}

void PCube::setMaxCurr(float maxCurr)
{
	ret=PCube_setMaxCur(0,this->id,maxCurr); 
}

float PCube::getMaxCurr()
{
	float temp;
	ret = PCube_getMaxCur(0, this->id, &temp);
	return temp;
}

PCube::PCube(int id)
{
	this->id = ID[id];
	this->inverted = false;
}

float PCube::getVelocity()
{
	float temp;
	ret = PCube_getVel(0, this->id, &temp);
	return temp;
}

float PCube::getActPos()
{
	float temp;
	ret=PCube_getPos(0,this->id,&temp); 
	return temp;
}

int PCube::getId()
{
	return this->id;
}

PCube::~PCube()
{

}
//
///////////////////////////////////////////////////////////////////////////////////////////////
//

class PlotterStage
{
	int id;					// ID of the Stage (only used to communicate with dll-s)
	int count;				// the number of PowerCubes in the system
	std::vector<PCube*> Cube;							// Pointers the PowerCube objects

public:
	PlotterStage();										// initializes the system
	~PlotterStage();									// clears the object
	
	// stage functions
	int Count();										// Returns with the number of powercubes found in the system
	void listMovingConstraints();						// printing the list of variables to sdtout

	// ID functions
	int getId(const int id);							// Gets the ID of the specified PCube

	// Homing functions
	void setHomeVel(const int id, float value);			// Sets the homing velocity of the specified PowerCube
	float getHomeVel(const int id);						// Get the homing velocity of the specified PCube

	void goHome(const int id);							// Moving to the home-position
	void goHomeAll();									// Moving all the PCubes to home position

	// Halt functions
	void haltAll();										// Stops all the cubes at the current position
	void halt(const int id);							// Stops the PCube

	// Reset functions
	void resetAll();									// resetting all the PCubes
	void reset(const int id);							// resetting the PCube

	// State functions
	unsigned long getState(const int id);	// Get the actual state of the powercube

	// Inversion functions
	void setInversion(int id, bool value);				// Sets the inversion of the PCubes coordinate-axis
	bool getInversion(int id);							// Gets the inversion value of the PCube's coordinate axis

	// Current functions
	void setMaxCurr(const int id, float value);			// Set the maximum allowed current on the specified PowerCube
	float getMaxCurr(const int id);						// Get the maximum allowed current on the specified PowerCube
	
	float getCurrent(const int id);						// Get the actual current flowing through the PCube
	void moveCurrent(const int id, float current);		// Moves the PCube with the value of the current flowing through it

	// Position functions
	void setMaxPos(const int id, float value);			// sets the maximum position of the PCube
	void setMinPos(const int id, float value);			// set the minimum position of the PCube
	float getMaxPos(const int id);						// get the maximum pos of the PCube
	float getMinPos(const int id);						// get the min. pos. of the PCube

	float getActPos(const int id);							// Get the actual position of the specified PCube
	void movePos(const int id, float value);			// Moving to the specified position
	void movePosExtended(const int id, float position, float velocity, float acceleration);

	// Velocity functions
	void setMaxVel(const int id, float velocity);			// Set the max. velocity of the PCube
	float getMaxVel(const int id);							// Get the max. velocity of the PCube

	float getVelocity(const int id);						// Gets the maxspeed of the specified PCube
	void moveVel(const int id, float velocity);				// Moves the PCube with the specified velocity

	// Acceleration functions
	void setMaxAcc(const int id, float value);				// Set the max acceleration value of the specified PCube
	float getMaxAcc(const int id);							// Get the max acceleration value of the specified PCube
};

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::listMovingConstraints()
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	printf("Moving constraints:\n");
	for(int i=0; i<this->count; i++)
		printf("%i [%f, %f]\n", i, this->getMinPos(i), this->getMaxPos(i));
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::moveVel(const int id, float velocity)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->moveVel(velocity);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setMaxVel(const int id, float velocity)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setMaxVel(velocity);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getMaxVel(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getMaxVel();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::movePosExtended(const int id, float position, float velocity, float acceleration)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->movePosExtended(position, velocity, acceleration);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setMaxPos(const int id, float value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setMaxPos(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setMinPos(const int id, float value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setMinPos(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getMaxPos(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getMaxPos();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getMinPos(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getMinPos();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::moveCurrent(const int id, float current)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->moveCur(current);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
int PlotterStage::getId(int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getId();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::goHomeAll()
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	for(int i=0; i<this->Count(); i++)
		this->Cube[i]->goHome();
}

// Throws int:
//		1: Init failed!
//		2: No PCubes in the system
PlotterStage::PlotterStage()
{
	this->id = 0;  // using random number as an ID
	this->Cube.clear();			// Clearing the pointer vector

	// Opening devices
	ret = PCube_openDevice( &(this->id), pInitString );
	
	// counting the number of PCubes in the system
	this->count = -1;
	this->count = PCube_getModuleCount(this->id);

	if (this->count < 0)
		throw new int(-1);
	if(this->count == -1)
		throw 1;
	if(this->count == 0)
		throw 2;

	// Creating the PCubes
	for(int i=0; i<this->count; i++)
		Cube.push_back(new PCube(i+1));

	// Resetting devices
	ret = PCube_resetAll(0);
}

PlotterStage::~PlotterStage()
{
	// Sending stop signals to the PCubes
	this->haltAll();

	ret = PCube_closeDevice(this->id);
}

// Throws 
//	4: not initialized 
int PlotterStage::Count()
{
	if(this->count == -1)
		throw 4;

	return this->count;
}

// Throws 
//	4: not initialized 
void PlotterStage::haltAll()
{
	if(this->count == -1)
		throw 4;

	ret = PCube_haltAll(0);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::halt(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->halt();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getActPos(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getActPos();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getVelocity(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getVelocity();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setMaxAcc(const int id, float value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setMaxAcc(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getMaxAcc(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getMaxAcc();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setHomeVel(const int id, float value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setHomeVel(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getHomeVel(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getHomeVel();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setMaxCurr(const int id, float value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setMaxCurr(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getMaxCurr(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getMaxCurr();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
float PlotterStage::getCurrent(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getMaxCurr();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::goHome(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->goHome();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
unsigned long PlotterStage::getState(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getModuleState();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::resetAll()
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	ret = PCube_resetAll(0);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::reset(const int id)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->reset();
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
//	5: MoveConstraint error
void PlotterStage::movePos(const int id, float value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->movePos(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
void PlotterStage::setInversion(int id, bool value)
{
	if(this->count == -1)
		throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	this->Cube[id]->setInversion(value);
}

// Throws
//	4: not initialized
//	3: Pcube ID out of range
bool PlotterStage::getInversion(int id)
{
	if(this->count == -1)
	throw 4;

	// Throwing PCube does not exist exception
	if(id < 0 || id >= this->count)
		throw 3;

	return this->Cube[id]->getInversion();
}
