#ifndef PLANSERVER_HPP
#define PLANSERVER_HPP

//****************************************************************************************************
//*                                              Includes                                            *
//****************************************************************************************************

using namespace System;
using namespace System::Net;
using namespace System::Net::Sockets;
using namespace System::Windows::Forms;

//****************************************************************************************************
//*                                       Declaration of the Class                                   *
//****************************************************************************************************
//name: Server
//description: The class responsibles for the undisturbed function of the network contacts and the 
// communication with the client
//----------------------------------------------------------------------------------------------------

ref class PlanServer{
	private:	
		TcpListener^ server;
		TcpClient^ client;
		TextBox^ textbox;

	public:
		PlanServer(TextBox^ _textbox);
		void Start();
		void Stop();
		void Tick();
		bool sendMSG(String^ str);
		String^ getMSG();
};

//****************************************************************************************************
//*                                      Definition of the Functions                                 *
//****************************************************************************************************

//----------------------------------------------------------------------------------------------------
//name: Server()
//return: -
//parameters: System::Windows::Forms::TextBox^(_textbox)
//description: Constructor with one parameter, which indicates the status of the connection
//----------------------------------------------------------------------------------------------------
PlanServer::PlanServer(TextBox^ _textbox){
	textbox=_textbox;
}
//----------------------------------------------------------------------------------------------------
//name: Start()
//return: -
//parameters: -
//description: The server start listen at port, and wait for client connection
//----------------------------------------------------------------------------------------------------
void PlanServer::Start(){
	server=gcnew TcpListener(IPAddress::Parse("10.3.19.20"),10101);
	server->Start();
	client=nullptr;
	textbox->Text="Waiting";
}

//----------------------------------------------------------------------------------------------------
//name: Stop()
//return: -
//parameters: -
//description: The server stop listen at port.
//----------------------------------------------------------------------------------------------------
void PlanServer::Stop(){
	server->Stop();
	client=nullptr;
}

//----------------------------------------------------------------------------------------------------
//name: Tick()
//return: -
//parameters: -
//description: While server is waiting, every timestamp check, is there client at port.
//----------------------------------------------------------------------------------------------------
void PlanServer::Tick(){
	if (client==nullptr){
		if (!server->Pending()) return;
		client=server->AcceptTcpClient();
		textbox->Text="Ready";
	}
}

//----------------------------------------------------------------------------------------------------
//name: sendMSG()
//return: bool
//parameters: System::String^(str)
//description: The function sends the given String to the client. If an error occurs, return false 
// else true.
//----------------------------------------------------------------------------------------------------
bool PlanServer::sendMSG(String^ str){
	try{
	if (client==nullptr) throw gcnew IO::IOException();
	array<Byte>^ data = System::Text::Encoding::ASCII->GetBytes(str);
	data->Clear(data,data->Length-1,1);
	client->GetStream()->Write(data,0,data->Length);
	client->GetStream()->Flush();
	return true;
	}catch (IO::IOException^ ex){
		ex;
		client->Close();
		client=nullptr;
		textbox->Text="Waiting";
		return false;
	}
}

//----------------------------------------------------------------------------------------------------
//name: getMSG()
//return: System::String^
//parameters: -
//description: The function receives message from the client. If no message arrived, return null.
//----------------------------------------------------------------------------------------------------
String^ PlanServer::getMSG(){
	try{
	if (client==nullptr) return nullptr;
	array<Byte>^ bytes=gcnew array<Byte>(128);
	Int32 i=0;
	i=client->GetStream()->Read(bytes,0,bytes->Length);
	String^ ret=System::Text::Encoding::ASCII->GetString(bytes,0,i);
	//textbox->Text=ret;
	return ret;	
	}catch (IO::IOException^ ex){
		ex;
		client->Close();
		client=nullptr;
		textbox->Text="Waiting";
		return nullptr;
	}
}

#endif