#include "stdafx.h"
#include "SensorInfo.h"

SensorInfo::SensorInfo()
{
	inputstat = InputStatus::No_Input;
	connstat = ConnStatus::None;
	BaudRate = 38400;
	parity = System::IO::Ports::Parity::None;
	DataBits = 8;
	stopbits = System::IO::Ports::StopBits::One;
	portname=L"COMinv";
	ReadBuffer = gcnew cli::array<unsigned char>(36);
	data_age=0;
}

InputStatus SensorInfo::GetInputStatus()
{
	return inputstat;
}

ConnStatus SensorInfo::GetConnStatus()
{
	return connstat;
}

ConnStatus SensorInfo::UsePort(System::String^ port)
{
	try{
		portname = port;
		serialport = gcnew System::IO::Ports::SerialPort(portname,BaudRate,parity,DataBits,stopbits);
		serialport->DataReceived += gcnew System::IO::Ports::SerialDataReceivedEventHandler(this, &SensorInfo::SerialDataReceived);
		serialport->ReadBufferSize = 36;
		serialport->Open();
		connstat = ConnStatus::Conn;
	}
	catch(System::IO::IOException ^e){
		connstat=ConnStatus::Error;
		inputstat=InputStatus::No_Input;
		e;
	}
	catch(System::UnauthorizedAccessException ^e){
		connstat=ConnStatus::Error;
		inputstat=InputStatus::No_Input;
		e;
	}
	return connstat;
}

System::Void SensorInfo::SerialDataReceived(System::Object ^sender,System::IO::Ports::SerialDataReceivedEventArgs ^e)
{
	try{
		int count=serialport->BytesToRead; //Available bytes
		for(int i=0; i<count; i++)
		{
			if(ReadBufferCounter<35){
				serialport->Read(ReadBuffer,ReadBufferCounter,1);
				ReadBufferCounter+=1;
			}
			else{
				serialport->Read(ReadBuffer,ReadBufferCounter,1);
				RefreshData();
				ReadBuffer->Initialize();
				ReadBufferCounter=0;
			}
		}
	}catch(System::InvalidOperationException^ exep){
		inputstat = InputStatus::Wrong;
		exep;
	}
}

System::Void SensorInfo::RefreshData()
{
	int iter=0;
	try
	{
		while(ReadBuffer[iter]!=55 || ReadBuffer[iter+1]!=66){
			iter+=1;
		}
	}catch(System::IndexOutOfRangeException^e){inputstat=InputStatus::Wrong; e;}

	if(iter<18 && ReadBuffer[iter+16]==11&&ReadBuffer[iter+17]==22){
		if(ReadBuffer[iter+2]==0 && ReadBuffer[iter+3] == 0)
		{
			distance1=400;
		}
		else
		{
			distance1 = 23559/(ReadBuffer[iter+2]*256+ReadBuffer[iter+3]);
		}
		if(distance1 > 400) distance1=400;

		if(ReadBuffer[iter+4]==0 && ReadBuffer[iter+5]==0)
		{
			distance2 = 400;
		}
		else
		{
			distance2 = 23559/(ReadBuffer[iter+4]*256+ReadBuffer[iter+5]);
		}
		if(distance2 > 400) distance2=400;
		
		LSB1 = ReadBuffer[iter+6]*256+ReadBuffer[iter+7];
		LSB2 = ReadBuffer[iter+8]*256+ReadBuffer[iter+9];
		LSB3 = ReadBuffer[iter+10]*256+ReadBuffer[iter+11];
		LSB4 = ReadBuffer[iter+12]*256+ReadBuffer[iter+13];
		LSBGrabb = ReadBuffer[iter+14]*256+ReadBuffer[iter+15];
		inputstat=InputStatus::OK;
		data_age = 0;
	}
	else{
		inputstat=InputStatus::Wrong;
	}
}

cli::array<int, 1>^ SensorInfo::GetSensorData()
{
	data_age+=1;
	cli::array<int, 1>^ data = gcnew cli::array<int>(7);
	data[0]=distance1;
	data[1]=distance2;
	data[2]=LSB1;
	data[3]=LSB2;
	data[4]=LSB3;
	data[5]=LSB4;
	data[6]=LSBGrabb;
	if(data_age > 10 && inputstat != InputStatus::No_Input)
	{
		inputstat=InputStatus::No_Input;
		connstat = ConnStatus::None;
		this->Close();
	}

	return data;
}

void SensorInfo::Close()
{
	 if(connstat==ConnStatus::Conn) serialport->Close();
}